/**
 * A basic, one-way Rustle template sketch.
 *
 * LICENSE: This source file is subject to the MIT License, which is available
 * through the World Wide Web at http://opensource.org/licenses/MIT, and is also
 * included as a part of this repository.
 *
 * @author     Andrew Hamon <and.ham95@gmail.com>
 * @author     Steven Petryk <petryk.steven@gmail.com>
 * @copyright  2014 Andrew Hamon
 * @license    MIT License
 */

#include <Adafruit_NeoPixel.h>

/*
 * Ensure that these directives match your physical setup.
 *
 * Note: The baud rate may need to be decreased depending on your setup. The Uno
 * seems to handle 921,600 just fine, and it's what Rustle defaults to.
 */
#define STRIP_LENGTH 30
#define STRIP_PIN 6
#define BAUD 921600

// Initialize LED strip
Adafruit_NeoPixel myStrip = Adafruit_NeoPixel(STRIP_LENGTH, STRIP_PIN,
  NEO_GRB + NEO_KHZ800);

// Current LED index within a frame
byte led_index;

// Each LED is an RGB array. `k` is just a counter (0,1,2  == R,G,B).
byte current_led[3];
byte k;

void setup() {
  // Ensure this matches the baud in your receiver initialization.
  Serial.begin(BAUD);

  // Initialize counters
  led_index = 0;
  k = 0;

  // Here we go
  myStrip.begin();
  myStrip.show();
}

void serialEvent() {
  while(Serial.available()) {
    byte x = Serial.read();

    // 255 is the end-of-frame code, so we can assume that all other values
    // are actual RGB values.
    if (x == 255) {
      myStrip.show();

      k = led_index = 0;
    }

    else if (x < 255) {
      // Data is sent [r,g,b,r,g,b,...] so we must accumulate an entire RGB
      // triplet before writing to the LED strip.
      current_led[k] = x;
      k++;

      // If we've accumulated a triplet...
      if(k > 2) {
        // ... we send the pixel.
        myStrip.setPixelColor(led_index, 
          current_led[0], current_led[1], current_led[2]);

        k = 0;
        led_index++;
      }
    }
  }
}

// Empty loop because sketch is entirely serial event driven
void loop() {};
